/**
 * Copyright (C) 2008  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.console.service.orchestra;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.Properties;
import java.util.Set;

import org.ow2.orchestra.facade.def.ProcessDefinition;
import org.ow2.orchestra.facade.def.ProcessState;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.facade.jmx.RemoteDeployerMBean;
import org.ow2.orchestra.facade.runtime.ProcessInstance;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.util.Misc;
import org.ow2.orchestra.util.OrchestraConstants;
import org.picocontainer.Startable;

/**
 * Manages interaction with Orchestra engine
 *
 * @author Le Jeune Goulven <goulven.le-jeune@bull.net>
 * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
 */
public class OrchestraService implements Startable {

  protected String jmxServiceUrl;
  protected String jmxObjectName;
  private boolean started = false;

  /*
   * (non-Javadoc)
   *
   * @see org.picocontainer.Startable#start()
   */
  public void start() {
  }

  public void init() {
    if (!this.started) {
      this.started = true;
      // Get the mbean server url
      final InputStream orchestraPropsStream = this.getClass().getClassLoader().getResourceAsStream("orchestra.properties");
      if (orchestraPropsStream == null) {
        throw new RuntimeException("orchestra.properties not found in classpath." + Misc.LINE_SEPARATOR);
      }
      final Properties orchestraProperties = new Properties();
      try {
        orchestraProperties.load(orchestraPropsStream);
      } catch (final Exception e) {
        // Default configuration not loaded
        throw new OrchestraRuntimeException("Exception occured while loading orchestra.properties", e);
      }
      this.jmxServiceUrl = orchestraProperties.getProperty(OrchestraConstants.JMX_SERVICE_URL);
      this.jmxObjectName = orchestraProperties.getProperty(OrchestraConstants.JMX_OBJECT_NAME);
    }
  }

  /*
   * (non-Javadoc)
   *
   * @see org.picocontainer.Startable#stop()
   */
  public void stop() {
    // TODO Auto-generated method stub

  }

  public void deployProcess(final InputStream is) throws IOException, DeploymentException {
    this.init();
    final byte[] barFile = Misc.getAllContentFrom(is);
    try {
      final RemoteDeployerMBean remoteDeployer = Misc.getMBeanProxy(RemoteDeployerMBean.class, this.jmxServiceUrl, this.jmxObjectName);
      remoteDeployer.deployBar(barFile);
    } catch (final Exception e) {
      throw new DeploymentException("Exception while deploying barFile: ", e);
    }
  }

  public void undeployProcess(final String pduuid) {
    this.init();

    try {
      final RemoteDeployerMBean remoteDeployer = Misc.getMBeanProxy(RemoteDeployerMBean.class, this.jmxServiceUrl, this.jmxObjectName);
      remoteDeployer.undeploy(new ProcessDefinitionUUID(pduuid));
    } catch (final Exception e) {
      throw new OrchestraRuntimeException("Exception caught while undeploying process : " + pduuid, e);
    }
  }

  public List<ProcessDefinition> getProcesses() {
    this.init();

    try {
      final RemoteDeployerMBean remoteDeployer = Misc.getMBeanProxy(RemoteDeployerMBean.class, this.jmxServiceUrl, this.jmxObjectName);
      final List<ProcessDefinition> processDefinitionDataList = remoteDeployer.findProcessDefinitions(ProcessState.DEPLOYED);
      return processDefinitionDataList;
    } catch (final Exception e) {
      throw new OrchestraRuntimeException(e);
    }

  }

  public ProcessDefinition getProcess(final String pduuid) {
    this.init();

    try {
      final RemoteDeployerMBean mbean = Misc.getMBeanProxy(RemoteDeployerMBean.class, this.jmxServiceUrl, this.jmxObjectName);
      final ProcessDefinition processDef = mbean.getProcessDefinition(new ProcessDefinitionUUID(pduuid));
      return processDef;
    } catch (final Exception e) {
      throw new OrchestraRuntimeException("Error while getting process definition " + pduuid, e);
    }
  }

  public Set<ProcessInstance> getInstances() throws DeploymentException {
    this.init();

    // get the instance list
    try {
      final RemoteDeployerMBean remoteDeployer = Misc.getMBeanProxy(RemoteDeployerMBean.class, this.jmxServiceUrl, this.jmxObjectName);
      return remoteDeployer.findProcessInstances();
    } catch (final Exception e) {
      throw new DeploymentException("Exception while getting process instances list", e);
    }
  }

  public Set<ProcessInstance> getInstances(final ProcessDefinitionUUID pduuid) throws DeploymentException {
    this.init();

    try {
      final RemoteDeployerMBean remoteDeployer = Misc.getMBeanProxy(RemoteDeployerMBean.class, this.jmxServiceUrl, this.jmxObjectName);
      return remoteDeployer.findProcessInstances(pduuid);
    } catch (final Exception e) {
      throw new DeploymentException("Exception while getting process instances list", e);
    }
  }

}
